import LGames.Context;
import LGames.Agent;
import LGames.CompositionalAgent2;
import LGames.HolisticAgent;
import LGames.HolisticAgent2;
import LGames.DGame;
import LGames.Meaning;
import LGames.Rules2;
import Interface.GeomJComponent;
import Interface.LanguageJComponent;
import Interface.ControlDialog;
import Util.Utils;
import LGames.Symbols;
import Util.Parameters;
import Interface.PopMenu;
import Interface.Statistics;
import Interface.CompositionalStatistics;
import Interface.Canvas;
import Util.Stats;
import Util.CompositionalStats;
import Util.HolisticStats;
import Util.DoubleArray;
import Util.IntArray;

import java.util.*;
import java.io.*;
import java.awt.Container;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.Graphics;
import java.awt.Color;
import javax.swing.*;
import javax.swing.JMenuBar;

/**
 * The main class for THSim v4.0.3
 *
 * <p>
 * Copyright (c) 2004 Paul Vogt
 *
 * @author Paul Vogt
 * @version $4.0.3$
 */

public class THSim{

    protected static Context cxt;
    protected static List lAgents = new ArrayList();
    protected static int nAgents = 2;
    protected final static double noiseCxt = 0.0;
    protected static Agent speaker;
    protected static Agent hearer;
    protected static int lastID=0;
    protected static Parameters parameters = new Parameters();
    private static boolean [] views = new boolean [3];
    protected static Stats pStats;
    protected static List lScoreFiles = new ArrayList();
    protected static List lWordList = new ArrayList();
    protected static List lWords = new ArrayList();


    /**
     * Initialise a list of agents
     */
    protected static void create_agents(){

	for (int i=0;i<parameters.getNAgents();i++){
	    switch (parameters.getAgentType()){
	    case 0:
		lAgents.add(new HolisticAgent(lastID,0,
					      parameters.getEtaN(),
					      parameters.getEtaS(),
					      parameters.getUpdateScore(),
					      parameters.getNMeanings(),
					      parameters.getNSymbols(),
					      parameters.getFeatures()));
		break;
	    case 1:
		lAgents.add(new HolisticAgent2(lastID,0,
					       parameters.getEtaN(),
					       parameters.getEtaS(),
					       parameters.getUpdateScore(),
					       parameters.getNMeanings(),
					       parameters.getNSymbols(),
					       parameters.getFeatures()));
		break;
	    case 2:
		lAgents.add(new CompositionalAgent2(lastID,0,
						    parameters.getEtaN(),
						    parameters.getEtaS(),
						    parameters.getFeatures(),
						    parameters.getNNoLearning(),
						    parameters.getAlphabetSize()));
		break;
	    }
	    lastID++;
	    if (parameters.getPrintScore()){ 
		lScoreFiles.add(new ArrayList());
		lWordList.add(new ArrayList());
	    }

	}
    }

    /**
     * Reinitialise the list of agents
     */
    protected static void reinitializeAgents(int lg){

	int maxAge=0;
	int oldest=-1;

	if (parameters.getNAgents()>lAgents.size()){
	    for (int i=lAgents.size();i<parameters.getNAgents();i++){
		switch (parameters.getAgentType()){
		case 0:
		    lAgents.add(new HolisticAgent(lastID,0,
						  parameters.getEtaN(),
						  parameters.getEtaS(),
						  parameters.getUpdateScore(),
						  parameters.getNMeanings(),
						  parameters.getNSymbols(),
						  parameters.getFeatures()));
		    break;
		case 1:
		    lAgents.add(new HolisticAgent2(lastID,0,
						   parameters.getEtaN(),
						   parameters.getEtaS(),
						   parameters.getUpdateScore(),
						   parameters.getNMeanings(),
						   parameters.getNSymbols(),
						   parameters.getFeatures()));
		    break;
		case 2:
		    lAgents.add(new CompositionalAgent2(lastID,0,
							parameters.getEtaN(),
							parameters.getEtaS(),
							parameters.getFeatures(),
							parameters.getNNoLearning(),
							parameters.getAlphabetSize()));
		    break;
		}
		lastID++;
		if (parameters.getPrintScore()){ 
		    lScoreFiles.add(new ArrayList());
		    lWordList.add(new ArrayList());
		}

	    }
	}
	else{
	    while (parameters.getNAgents()<lAgents.size()){
		maxAge=1000000000;
		oldest=-1;
		for (int i=0;i<lAgents.size();i++){
		    if (((Agent)lAgents.get(i)).getID()<=maxAge){
			maxAge=((Agent)lAgents.get(i)).getID();
			oldest=i;
		    }
		}
		if (oldest>=0){
		    lAgents.remove(oldest);
		    if (parameters.getPrintScore()){ 
			lScoreFiles.remove(oldest);
			lWordList.remove(oldest);
		    }
		}
	    }
	}
    }

    private static void replaceHolistics(){
	List tmpList = new ArrayList();
	for (int i=0;i<lAgents.size();i++){//for the moment only into agents of type 2
	    tmpList.add(new CompositionalAgent2((HolisticAgent)lAgents.get(i),parameters.getFeatures(),
						parameters.getNNoLearning(),
						parameters.getAlphabetSize()));
	}
	lAgents=tmpList;
    }

    /**
     * The function test() can be used to test any function specified inside. <br>
     * This function was used for testing and debugging the system. <br>
     * The example function is used to give a speaker a predefined  <br>
     * compositional language, which is then learnt by a hearer. <br>
     */
   
    protected static void test() throws IOException{

	List cs = new ArrayList();
	speaker=new CompositionalAgent2(true,cs);
	final List CS = cs;
	System.out.println(CS);


	List dummyList=null;
	hearer=new CompositionalAgent2(false,dummyList);

	
	int t=0;
	int s=0;
	int h=1;
	Symbols utteranceSpeaker;
	Symbols interpretationHearer;

	pStats=new CompositionalStats();

	File logFile = null;
	boolean openedLogFile=false;
	boolean openedLexFile=false;

	if (parameters.getLogFile()){
	    parameters.openFile(parameters.getLogFileName());
	    openedLogFile=true;
	}

	if (parameters.getSaveLexicon()){
	    parameters.openLexIIFile(parameters.getLexIIFileName());
	    //openedLexFile=true;
	}

	if (parameters.getPrintLexicon()){
	    parameters.openLexFile(parameters.getLexFileName());
	    // openedLexFile=true;
	}

	int ngames=parameters.getNRuns();
	int nIter=parameters.getNIterations();

	boolean run=false;

	for (int iter=0;iter<nIter;iter++)
	    for (int i=0;i<ngames;i++){
		t=(int)Math.round(CS.size()*Math.random()-0.50001);
		
		if (t<0) t=0;
		IntArray DCS=new IntArray((IntArray)CS.get(t));
		if (DCS.isEmpty()) Utils.error("dcs is empty, snik, t="+t+": "+CS);

		((CompositionalAgent2)speaker).setDCS(DCS);
		((CompositionalAgent2)hearer).setDCS(DCS);
		
		System.out.println("lg="+i+" t="+t+" DCS="+DCS);
		
		//speaker.setTopicID(t);
		speaker.setTopic(t);
		//hearer.setTopicID(t);
		hearer.setTopic(t);
		
		utteranceSpeaker=speaker.
		    speak(parameters.getCreationProb(),
			  parameters.getUpdateScore());
		
		System.out.println("utterance="+utteranceSpeaker);
		
		interpretationHearer=hearer.hear(utteranceSpeaker,
		 			 parameters.getUpdateScore());

		System.out.println("interpretation="+interpretationHearer);
			
		speaker.adaptLexiconSpeaker(interpretationHearer,
					    hearer.getTopic(),
					    parameters.getGameType(),
					    parameters.getUpdateScore());
		
		hearer.adaptLexiconHearer(utteranceSpeaker,
				  speaker.getTopic(),
				  parameters.getGameType(),
				  parameters.getUpdateScore());
		
		pStats.update(speaker,hearer,parameters.getAgentType());
		
		if (parameters.getLogFile()){
		    
		    if (parameters.getFile()==null)
			if (logFile != null)
			    parameters.openFile(logFile);
			else
			    parameters.
				openFile(parameters.getLogFileName());
		    if (parameters.getFile()!=null){
			pStats.logFile(parameters.getFile());
			parameters.flushFile();
			
		    }
		    
		    
		}
		if (i%ngames==ngames-1){
		    System.out.println("speaker: ");
		    ((CompositionalAgent2)speaker).printGrammar();
		    System.out.println("hearer: ");
		    ((CompositionalAgent2)hearer).printGrammar();

		    if (iter<nIter){
			speaker=new CompositionalAgent2((CompositionalAgent2)hearer);
			dummyList=null;
			hearer=new CompositionalAgent2(((CompositionalAgent2)speaker).getOntology());
		    }
		}
	    }



	parameters.closeFile();
	parameters.closeLexIIFile();
    }

    /**
     * testPopulation(int it) is used to test the quality of the population.
     * <p>
     * In short, a fixed number of games are set up (i.e. different contexts). <br>
     * In `each' game, each agent produces an utterance. In turn, <br>
     * each agent tries to interpret each utterance. During this test-phase <br>
     * no new knowledge is invented or learnt. 
     * <p>
     * The output is in the form:
     * <p>
     * holistic agent type 0: <br>
     * "TP it pCoh iCoh var pCohAdults iCohAdults pCohLearners iCohLearners" <br>
     * holistic agent type 1: <br>
     * "TP it pCoh iCoh pCohAdults iCohAdults pCohLearners iCohLearners" <br>
     * compositional agent type 2: <br>
     * "TP it pCoh iCoh comp simSyntax pCohAdults iCohAdults pCohLearners iCohLearners" <br>
     * <p>
     * where: <br>
     *  TP is the identifier used to extract the test results from the standard output. <br>
     *  it is the iteration number <br>
     *  pCoh is the overal production coherence <br>
     *  iCoh is the interpretation accuracy <br>
     *  var is the average variance between meanings <br>
     *  comp is the compositionality measure <br>
     *  simSyntax is the similarity measure comparing the agents' grammars <br>
     *  pCohAdults is the coherence inside the adults population (i.e. the first half of the pop.) <br>
     *  iCohAdults is the accuracy inside the adults population <br>
     *  pCohLearners is the coherence inside the learners population <br>
     *  iCohLearners is the accuracy inside the learners population <br>
     */

    protected static void testPopulation(int it){
	//testing the quality of a population, 
	//writes output to standard output, preceded by testpop
	//currently only for holistic agents

	System.out.println("info testing population");
	double split=(double)lAgents.size()/2.;
	int sumPrdctn=0,sumIntrprtn=0;
	int sumCompositionality=0;
	int sComp=0;
	Rules2 [][] rule = new Rules2 [lAgents.size()][lAgents.size()];
	int [] sPrdctn = new int [2];
	int [] sIntrprtn = new int [2];
	String [] utterance = new String [lAgents.size()];
	int [][] interpretation = new int [lAgents.size()][lAgents.size()];
	int group;
	int dim=0;
	boolean [] features=parameters.getFeatures();
	for (int i=0;i<features.length;i++) if (features[i]) dim++;
	double [][] meaning = new double [lAgents.size()][dim];
	double [] nullMeaning = new double [dim];
	for (int i=0;i<dim;i++) nullMeaning[i]=-1.;
	double varMeaning=0.0;
	final int testSize = 200;
	Symbols utteranceSpeaker,interpretationHearer;
	int layer;
	int sum=0;
	int [] s=new int [2];
	int [] tSum=new int [2];

	for (int n=0;n<testSize;n++){
	    //select context
	    cxt=new Context(parameters.getMaxCxtSize(),
			    features,parameters.getFixedColours());
	    //select topic, each agent uses same topic
	    int topic=(int)Math.round((double)(parameters.
					       getMaxCxtSize()*Math.random())-0.50001);
	    if (topic<0) topic=0;
	    cxt.setFOA(topic,parameters.getFOA());

	    for (int i=0;i<lAgents.size();i++){
		utteranceSpeaker=null;
		//production
		if (parameters.getAgentType()==1){//sorry for the weird order
		    speaker=(HolisticAgent2)lAgents.get(i);
		    speaker.setContext(cxt,parameters.getNoiseCxt());
		    speaker.setTopic(topic);
		    layer=1;
		    //playing discr.games in layers until production succeeds.
		    while (utteranceSpeaker==null && layer<=speaker.getMaxLayer()){
			speaker.playDGame(parameters.getGameType(),
					  parameters.getUpdatePType(),layer,n,false);
			utteranceSpeaker=speaker.
			    speak(0.0,parameters.getUpdateScore());
			layer++;
		    }
		    //acuire some data
		    Meaning m=((HolisticAgent2)speaker).getTOPIC();
		    if (m!=null)
			meaning[i]=m.getPrototype();
		    else meaning[i]=nullMeaning;
		    if (utteranceSpeaker!=null)
			utterance[i]=utteranceSpeaker.getForm();
		    else
			utterance[i]=null;
		}
		else if (parameters.getAgentType()==0){
		    speaker=(HolisticAgent)lAgents.get(i);
		    speaker.setContext(cxt,parameters.getNoiseCxt());
		    speaker.setTopic(topic);
		    //playing discr.game only one layer
		    speaker.playDGame(parameters.getGameType(),
				      parameters.getUpdatePType(),n,false);
		    //produce utterance
		    utteranceSpeaker=speaker.
			speak(0.0,parameters.getUpdateScore());
		    Meaning m=((HolisticAgent)speaker).getTOPIC();
		    if (m!=null)
			meaning[i]=m.getPrototype();
		    else meaning[i]=nullMeaning;
		    if (utteranceSpeaker!=null)
			utterance[i]=utteranceSpeaker.getForm();
		    else utterance[i]=null;
		}
		else if (parameters.getAgentType()==2){
		    //compositional agent
		    speaker=(CompositionalAgent2)lAgents.get(i);
		    speaker.setContext(cxt,parameters.getNoiseCxt());
		    speaker.setTopic(topic);
		    speaker.playDGame(parameters.getGameType(),
				      parameters.getUpdatePType(),n,false);
		    utteranceSpeaker=((CompositionalAgent2)speaker).
			speak(0.0,parameters.getUpdateScore(),false);
		    if (utteranceSpeaker!=null){
			utterance[i]=utteranceSpeaker.getForm();
			rule[i][i]=((CompositionalAgent2)speaker).getRule();
		    }
		    else{ 
			utterance[i]=null;
			rule[i][i]=null;
		    }
		}
		//each agent interprets the just produced utterance
		if (parameters.getAgentType()==0 || parameters.getAgentType()==1){
		    int u=-1;
		    int k=0;
		    while (u<0){
			if (utterance[k]!=null){ 
			    if (utterance[k].equals(utterance[i])) u=k;
			}
			else if (utterance[i]==null) u=k;
			k++;
		    }
		    
		    if (u<i) interpretation[i]=interpretation[u];
		    else{
			for (int j=0;j<lAgents.size();j++){
			    if (parameters.getAgentType()==1){
				hearer=(HolisticAgent2)lAgents.get(j);
				hearer.setContext(cxt,parameters.getNoiseCxt());
				interpretationHearer=null;
				
				layer=1;
				while (interpretationHearer==null && 
				       layer<=hearer.getMaxLayer()){

				    hearer.initDGame();
				    if (parameters.getGameType()=='o'){
					hearer.setTopic(topic);
					hearer.playDGame(parameters.getGameType(),
							 parameters.getUpdatePType(),
							 layer,n,false);
					interpretationHearer=hearer.hear(utteranceSpeaker,
									 parameters.
									 getUpdateScore());
				    }
				    else{
					for (int t=0;t<cxt.distinctive.length;t++){
					    hearer.setTopic(t);
					    hearer.playDGame(parameters.getGameType(),
							     parameters.getUpdatePType(),
							     layer,n,false);
					}
					hearer.setTopic(-1);
					interpretationHearer=hearer.
					    guess(utteranceSpeaker,
						  parameters.getUpdateScore());
				    }
				    layer++;
				}
			    }
			    else{
				hearer=(HolisticAgent)lAgents.get(j);
				hearer.setContext(cxt,parameters.getNoiseCxt());
				interpretationHearer=null;
				if (parameters.getGameType()=='o'){
				    hearer.setTopic(topic);
				    hearer.playDGame(parameters.getGameType(),
						     parameters.getUpdatePType(),n,false);
				    interpretationHearer=hearer.
					hear(utteranceSpeaker,
					     parameters.getUpdateScore());
				}
				else{
				    for (int t=0;t<cxt.distinctive.length;t++){
					hearer.setTopic(t);
					hearer.playDGame(parameters.getGameType(),
							 parameters.getUpdatePType(),n,false);
				    }
				    hearer.setTopic(-1);
				    interpretationHearer=hearer.
					guess(utteranceSpeaker,
					      parameters.getUpdateScore());
				}
			    }
			    if (interpretationHearer!=null) 
				interpretation[i][j]=hearer.getTopic();
			    else interpretation[i][j]=-1;
			}
			
		    }
		}
		else if (parameters.getAgentType()==2){
		    //The compositional agents
		    for (int j=0;j<lAgents.size();j++){
			if (j!=i){
			    hearer=(CompositionalAgent2)lAgents.get(j);
			    if (utteranceSpeaker!=null){
				if (parameters.getGameType()=='o'){
				    hearer.setContext(cxt,parameters.getNoiseCxt());
				    hearer.setTopic(topic);
				    hearer.playDGame(parameters.getGameType(),
						     parameters.getUpdatePType(),n,false);
				    interpretationHearer=((CompositionalAgent2)hearer).
					hear(utteranceSpeaker,
					     parameters.getUpdateScore(),false);
				}
				else{
				    hearer.setContext(cxt,parameters.getNoiseCxt());
				    ((CompositionalAgent2)hearer).initDCS();
				    for (int t=0;t<parameters.getMaxCxtSize();t++){
					hearer.setTopic(t);
					hearer.playDGame(parameters.getGameType(),
							 parameters.getUpdatePType(),n,false);
				    }
				    interpretationHearer=((CompositionalAgent2)hearer).
					guess(utteranceSpeaker,
					      parameters.getUpdateScore(),false);
				}
			    }
			    else interpretationHearer=null;
			    if (interpretationHearer!=null){
				if (parameters.getGameType()=='o')
				    interpretation[i][j]=topic;
				//we know this for sure as they play observational game
				else interpretation[i][j]=hearer.getTopic();
				rule[i][j]=((CompositionalAgent2)hearer).getRule();
			    }
			    else{
				interpretation[i][j]=-1;
				rule[i][j]=null;
			    }
			}
		    }
		}
	    }

	    //calculate measures
	    //distnguish between holistic agents and compositional agents
	    //sorry, no further comments
	    if (parameters.getAgentType()==2){
		for (int i=0;i<lAgents.size();i++){
		if (utterance[i]!=null){
		    sumCompositionality+=rule[i][i].compositionality();
		    sComp++;
		    for (int j=0;j<lAgents.size();j++){
			if (j!=i && utterance[i].equals(utterance[j])){
			    sumPrdctn++;
			    if (j<split)
				sPrdctn[0]++;
			    else sPrdctn[1]++;
			}
			if (j!=i && interpretation[i][j]==topic){ 
			    sumIntrprtn++;
			    if (j<split)
				sIntrprtn[0]++;
			    else sIntrprtn[1]++;
			    sumCompositionality+=rule[i][j].compositionality();
			    sComp++;
			}
		    }
		}
		}
	    }
	    else{
		int u=-1;
		int maxSum=0;
		for (int i=0;i<lAgents.size();i++){
		    if (utterance[i]!=null && (u<0 || !utterance[i].equals(utterance[u]))){
			sum=0;s[0]=0;s[1]=0;
			for (int j=0;j<lAgents.size();j++)
			if (i!=j && utterance[i].equals(utterance[j])){ 
			    sum++;
			    if (j<split) s[0]++;
			    else s[1]++;
			}
			if (sum>maxSum){
			    maxSum=sum;
			    u=i;
			    tSum[0]=s[0];
			    tSum[1]=s[1];
			}
		    }
		}
		varMeaning+=Utils.variance(meaning,lAgents.size(),dim);
		sumPrdctn+=maxSum;
		sPrdctn[0]+=tSum[0];
		sPrdctn[1]+=tSum[1];
		if (u>=0){
		    for (int i=0;i<lAgents.size();i++){
			if (interpretation[u][i]==topic){
			    sumIntrprtn++;
			    if (i<split) sIntrprtn[0]++;
			    else sIntrprtn[1]++;
			}
		    }
		}
	    }
	}

	double denom1=(double)(lAgents.size()*testSize);
	double denom2=((double)lAgents.size()/2.)*(double)testSize;
	double denom3=(double)(lAgents.size()*(lAgents.size()-1)*testSize);
	double denom4=(double)((lAgents.size()-1)*testSize);
	double pCoherence;
	double iCoherence;
	double var=1.0;
	if (parameters.getAgentType()==2){
	    pCoherence=(double)sumPrdctn/(denom3);
	    iCoherence=(double)sumIntrprtn/denom3;
	}
	else{
	    pCoherence=(double)sumPrdctn/(denom4);
	    iCoherence=(double)sumIntrprtn/denom1;
	    //System.out.print("varMeaning="+varMeaning);
	    var=varMeaning/(double)testSize;
	    System.out.println(" var="+var);
	}
	System.out.print("TP "+it+" "+Utils.doubleString(pCoherence,5)+" "+
			 Utils.doubleString(iCoherence,5)+" ");
	double comp=0.0;
	if (parameters.getAgentType()==2){
	    if (sComp>0)
		comp=(double)sumCompositionality/(double)sComp;
	    double simSyntax=((CompositionalAgent2)hearer).
		compareAgents((CompositionalAgent2)speaker);
	    System.out.print(Utils.doubleString(comp,5)+" "+
			     Utils.doubleString(simSyntax,5)+" ");
	}
	else
	    System.out.print(Utils.doubleString(var,5)+" ");
	for (int i=0;i<2;i++){
	    if (parameters.getAgentType()==2)
		System.out.print(Utils.doubleString((double)sPrdctn[i]/(0.5*denom3),5)+" "+
				 Utils.doubleString((double)sIntrprtn[i]/(0.5*denom3),5)+" ");
	    else
		System.out.print(Utils.doubleString((double)sPrdctn[i]/denom2,5)+" "+
				 Utils.doubleString((double)sIntrprtn[i]/denom2,5)+" ");
	}
	System.out.println();
	if (parameters.getAgentType()==0 || parameters.getAgentType()==1){
	    ((HolisticStats)pStats).updateCoherence(pCoherence,iCoherence);
	}
	else ((CompositionalStats)pStats).updateCoherence(pCoherence,iCoherence,comp);
    }

    /**
     * The main function of THSim. Here is where it all starts.
     * <p>
     * THsim starts with initialising all kinds of things, such as setting the parameters
     * following those given, or following the default settings.
     * The parameters are mostly stored in the Utils.Parameters class.
     * The interactive canvas is initialised when required.
     * <p>
     * Then, the program waits for a start-signal. This may be given as an argument,
     * or from the user-interface.
     * <p>
     * Given the start signal, the simulation starts, possibly after (re)initialising 
     * some data structures.
     * <p>
     * During the simulation, output is written to the standard output, which I usually
     * redirect into a file for further analysis (only for linux users though).
     * If the user interface is active, additional output is written there. Further
     * output can be stored in files as specified with the arguments.
     */

    public static void main(final String[] pArgs) throws IOException{

	Canvas canvas = null;
	parameters.parseParameters(pArgs);

	if (parameters.getTest()){
	    
	    test();
	    
	    System.exit(0);
	    
	}

	create_agents();
	
	//initialising statistics class
	if (parameters.getAgentType()==2)
	    pStats=new CompositionalStats();
	else
	    pStats = new HolisticStats();
	
	if (parameters.getVisible())
	    canvas = new Canvas(pStats,parameters);

	int N=0;
	Symbols utteranceSpeaker=null;
	Symbols interpretationHearer=null;
	int nrRuns=0;
	File logFile = null;
	boolean openedLogFile=false;
	boolean openedLexFile=false;

	if (parameters.getLogFile()){
	    parameters.openFile(parameters.getLogFileName());
	    openedLogFile=true;
	}

	if (parameters.getPrintLexicon()){
	    parameters.openLexFile(parameters.getLexFileName());
	    openedLexFile=true;
	}
	if (parameters.getSaveLexicon()){
	    parameters.openLexIIFile(parameters.getLexIIFileName());
	}

	parameters.setNewGame(true);

	int nAdults=0;
	int nLearners=0;

	char updateScore='s';
	int ns=0,nh=1;
	int iter=0;
	int layer=0;
	int [][] trainingSet = new int [0][0];
	int oldAgentType=parameters.getAgentType();

	while (!parameters.getQuit()){

	   
	    //when new game is started, new initialisations.
	    //possibly no longer necessary, because redone at later stage?
	    if (parameters.getNewGame()){
		lScoreFiles.clear();
		lWordList.clear();
		parameters.setNewGame(false);
		lAgents.clear();
		create_agents();
		if (parameters.getAgentType()==2) 
		    pStats=new CompositionalStats();
		else pStats=new HolisticStats();
		oldAgentType=parameters.getAgentType();
		nrRuns=0;N=0;
		logFile=null;
		iter=0;
		if (parameters.getFile()!=null)
		    parameters.killFile();
		if (parameters.getVisible())
		    canvas.updateNewGame(pStats,parameters.getAgentType());
	    }

	    nAgents=parameters.getNAgents();

	    //starting the function to display the lexicon or grammar
	    //This can only be called from the UI, and when the simulation 
	    //is not running.
	    //The simulation cannot be continued after the lexicon/grammar is closed
	    if (parameters.getShowLexicon()){
		canvas.showLexicon(lAgents,parameters.getAgentType(),
				   parameters.getUpdateScore());
		parameters.setShowLexicon(false);
	    }

	    //if getRun -> run simulation for nGames
	    //if getStep -> run simulation for nStep games
	    if (parameters.getRun() || parameters.getStep()){

		if (!parameters.getILM()) iter=0;

		//if the agent type is changed, reinitialise the whole bit.
		if (parameters.getAgentType()!=oldAgentType){
		    if (parameters.getAgentType()==2) 
			pStats=new CompositionalStats();
		    else pStats=new HolisticStats();
		    oldAgentType=parameters.getAgentType();
		    lAgents.clear();
		    create_agents();
		}

		//if we want to keep track of the scores, reinitialise this bit
		if (updateScore!=parameters.getUpdateScore() && N==0){
		    lAgents.clear();
		    lScoreFiles.clear();
		    lWordList.clear();
		    create_agents();
		    updateScore=parameters.getUpdateScore();
		}

		//for regulating incremental growth
		if (nAgents+parameters.getPopGrowth()>parameters.getMaxAgents() &&
		    parameters.ilmIncremental()){
		    if (parameters.getVisible()){
			canvas.errorMessage(1);
			parameters.setStop(true);
		    }
		    else{ 
			System.out.println("nAgents+popGrowth should be <= maxAgents");
			System.exit(0);
		    }
		}
		//foa is focus of attention, should be smaller than context size.
		if (parameters.getFOA() > parameters.getMaxCxtSize()){

		    if (parameters.getVisible()){
			canvas.errorMessage(2);
			parameters.setStop(true);
		    }
		    else{
			System.out.println("foa should be <= cxtSize");
			System.exit(0);
		    }
		}
		
		//iterate for nIterations or until quit, stop or newGame is pressed.
		while (iter<parameters.getNIterations() && 
			 !parameters.getQuit() && 
			 !parameters.getStop() &&
			 !parameters.getNewGame()){

		    //setting the bottleneck
		    if (parameters.trainingSet()>0)
			trainingSet=(new Context()).setTrainingSet(parameters.trainingSet());

		    //Again some initialisations, necessary in iterated learning model
		    //throwing away adult population, assigning learners to new adults,
		    //and assigning new fresh agents as learners.
		    if(parameters.getILM() && N==nrRuns){

			if (iter==0 && parameters.getSpeakerToSpeaker() && N==0){
			    if (parameters.ilmIncremental())
				nAdults=nAgents/2;
			    else nAdults=nAgents;
			    nLearners=nAdults;
			    parameters.setNAgents(nAgents);
			    reinitializeAgents(0);
			    pStats.reinitialize('i');
			}
			else if (N%parameters.getNRuns()==0){
			    if (parameters.getHolistic()){
				if (iter==0){
				    oldAgentType=parameters.getAgentType();
				    parameters.setAgentType(0);
				    lAgents.clear();
				}
				else if (iter==1){
				    parameters.setAgentType(oldAgentType);
				    replaceHolistics();
				}
			    }
			    if (parameters.ilmIncremental()){
				nAdults=nLearners;
				nLearners=Math.min(parameters.getMaxAgents()-
						   nAdults,nLearners+
						   parameters.getPopGrowth());
			    }
			    else{
				nAdults=nAgents/2;
				nLearners=nAgents-nAdults;
			    }
			    nAgents=nAdults+nLearners;
			    parameters.setNAgents(nAdults);
			    reinitializeAgents(N);
			    parameters.setNAgents(nAgents);
			    reinitializeAgents(N);
			    pStats.reinitialize('i');

			}
		    }
		    else if (parameters.getNAgents()!=lAgents.size())
			reinitializeAgents(N);
		
		    //assigning probabilities with which aadults can be selected
		    //as speaker and hearer
		    double pAdultSpeaker=parameters.getPAdultSpeaker();
		    double pAdultHearer=parameters.getPAdultHearer();

		    //step or run
		    if (parameters.getStep()) nrRuns=N+parameters.getStepSize();
		    else nrRuns=(iter+1)*parameters.getNRuns();

		    System.out.println("info nIter="+parameters.getNIterations()+" nRuns="+nrRuns+
				       " iter="+iter+" lg="+N+" popSize="+lAgents.size()+
				       " nAdults="+nAdults+" nLearners="+nLearners);

		    //here we go....
		    for (int n=N;n<nrRuns && !parameters.getStop() 
			     && !parameters.getQuit();n++){

			//create context
			if (parameters.trainingSet()>0)//with bottleneck
			    cxt = new Context(parameters.getMaxCxtSize(),
					      parameters.getFeatures(),
					      parameters.getFixedColours(),
					      trainingSet,parameters.trainingSet());
			else
			    cxt = new Context(parameters.getMaxCxtSize(),
					      parameters.getFeatures(),
					      parameters.getFixedColours());
			int h=0,s=1;
			//assigning the speaker and hearer
			if (iter==0 && parameters.getSpeakerToSpeaker()){
			    //in this case: both speakers and hearers from
			    //total population
			    s=(int)Math.round((double)parameters.getNAgents()*
					      Math.random()-0.50001);

			    h = s;
			    while (h == s){
				h=(int)Math.round((double)parameters.
						  getNAgents()*
						  Math.random()-0.50001);
				if (h<0) h=0;
			    }
			}
			else{
			    //in this case: speaker from adult population (or with a certain
			    //prob. from learner population).
			    //hearer from learner population (or with certain prob. from 
			    //adult population).
			    if (Math.random()<=pAdultSpeaker){
				s=(int)Math.round(nAdults*
						  Math.random()-0.50001);
				if (s<0) s=0;
			    }
			    else{
				s=(int)Math.round(nLearners*
						  Math.random()-0.50001);
				if (s<0) s=0;
				s+=nAdults;
			    }
			    if (Math.random()<pAdultHearer){
				h = s;
				while (h == s){
				    h=(int)Math.round(nAdults*
						      Math.random()-0.50001);
				    if (h<0) h=0;
				}
			    }
			    else{
				h=s;
				while (h==s){
				    h=(int)Math.round(nLearners*
						      Math.random()-0.50001);
				    if (h<0) h=0;
				    h+=nAdults;
				}
			    }
			    //just in case:
			    if (s==h) Utils.error("something went wrong");
			}
			if (s<0) s=0;

			//initialising speaker and hearer with right class
			switch(parameters.getAgentType()){
			case 0:
			    speaker = (HolisticAgent)lAgents.get(s);
			    hearer = (HolisticAgent)lAgents.get(h);
			    break;
			case 1:
			    speaker = (HolisticAgent2)lAgents.get(s);
			    hearer = (HolisticAgent2)lAgents.get(h);
			    break;
			case 2:
			    speaker = (CompositionalAgent2)lAgents.get(s);
			    hearer = (CompositionalAgent2)lAgents.get(h);
			    break;
			}

			//selecting topic
			int t=(int)Math.round(
				    (double)cxt.distinctive.length*
				    Math.random()
				    -0.50001);
			if (t<0) t=0;
			speaker.setTopic(t);

			//setting the focus of attention
			cxt.setFOA(t,parameters.getFOA());

			//adding perceptual noise to the features
			speaker.setContext(cxt,parameters.getNoiseCxt());

			//producing utterances
			if (parameters.getAgentType()==1){
			    //hierarchical, holistic agent
			    utteranceSpeaker=null;
			    layer=1;
			    double prob=0.0;
			    //play a discr.game at different layers until
			    //utterance produced or layers run out.
			    while (utteranceSpeaker==null && layer<=speaker.getMaxLayer()){
				speaker.playDGame(parameters.getGameType(),
						  parameters.getUpdatePType(),layer,n,true);
				//inventing new words only at final layer.
				if (layer==speaker.getMaxLayer()) 
				    prob=parameters.getCreationProb();
				utteranceSpeaker=speaker.
				    speak(prob,
					  parameters.getUpdateScore());
				layer++;
			    }
			}
			else{
			    //holistic and compositional agent
			    //play a discr. game
			    speaker.playDGame(parameters.getGameType(),
					      parameters.getUpdatePType(),n,true);
			    //produce an utterance
			    utteranceSpeaker=speaker.
				speak(parameters.getCreationProb(),
				      parameters.getUpdateScore());
			}

			hearer.setContext(cxt,noiseCxt);

			//if there is an utterance, the hearer has to act!
			if (utteranceSpeaker!=null){
			    if (parameters.getVariation()){
				//If we want to vary the type of language game during
				//a run, we select the type here
				
				double select=Math.random();
				
				if (select<=parameters.getOGProbability())
				    parameters.setGameType('o');
				else if (select<=parameters.getOGProbability()+
					 parameters.getGGProbability())
				    parameters.setGameType('g');
				else parameters.setGameType('s');
			    }
			    
			    if (parameters.getGameType()=='o'){//observational game
				//establishing joint attention
				hearer.setTopic(speaker.getTopic());
				if (parameters.getAgentType()==1){
				    //hierarchical type
				    interpretationHearer=null;
				    layer=1;
				    //play discr.game& interpret at increasing layers
				    //until interpretation succeeds or maxLayers reached.
				    while (interpretationHearer==null && 
					   layer<=hearer.getMaxLayer()){
					hearer.
					    playDGame('o',parameters.getUpdatePType(),
						      layer,n,true);
					interpretationHearer=hearer.
					    hear(utteranceSpeaker,
						 parameters.getUpdateScore());
					layer++;
				    }
				}
				else{
				    //same for holistic & compositional agent 
				    hearer.playDGame('o',parameters.getUpdatePType(),n,true);
				    interpretationHearer=hearer.
					hear(utteranceSpeaker,
					     parameters.getUpdateScore());
				}
			    }
			    else {//guessing or selfish game
				if (parameters.getAgentType()==1){
				    //our hierarchical agent
				    interpretationHearer=null;
				    layer=1;
				    while (interpretationHearer==null && 
					   layer<=hearer.getMaxLayer()){
					//resetting the distintiveness of categories
					hearer.initDGame();
					for (t=0;t<cxt.distinctive.length;t++){
					    hearer.setTopic(t);
					    hearer.playDGame(parameters.getGameType(),
							     parameters.getUpdatePType(),
							     layer,n,true);
					}
					hearer.setTopic(-1);
					interpretationHearer=hearer.
					    guess(utteranceSpeaker,
						  parameters.getUpdateScore());
					layer++;
				    }
				}
				else{
				    if (parameters.getAgentType()==2)
					//making sure the dist.cat.set is empty
					((CompositionalAgent2)hearer).initDCS();
				    for (t=0;t<cxt.distinctive.length;t++){
					hearer.setTopic(t);
					hearer.playDGame(parameters.getGameType(),
							 parameters.getUpdatePType(),n,true);
				    }
				    hearer.setTopic(-1);
				    interpretationHearer=hearer.
					guess(utteranceSpeaker,
					      parameters.getUpdateScore());
				}
			    }
			    //adaptation of scores
			    speaker.adaptLexiconSpeaker(interpretationHearer,
							hearer.getTopic(),
							parameters.getGameType(),
							parameters.getUpdateScore());
			    //adaptation of scores, and in case of holistic guys,
			    //adoption of words.
			    //not, however, if adaptSG=false and playing a selfish game.
			    if (parameters.getAdaptation() || 
				parameters.getGameType() != 's')
				hearer.adaptLexiconHearer(utteranceSpeaker,
							  speaker.getTopic(),
							  parameters.getGameType(),
							  parameters.getUpdateScore());
			}
			else{ 
			    hearer.setTopic(-1);
			    if (parameters.getAgentType()==2)
				((CompositionalAgent2)hearer).setRule(-1);
			}

			//updating the statistics
			if (parameters.getAgentType()==2)
			    ((CompositionalStats)pStats).
				update(speaker,hearer,parameters.getAgentType());
			else ((HolisticStats)pStats).
				 update(speaker,hearer,parameters.getAgentType());

			//if we want to keep track of scores, this is where it happens
			//produces many files for some words, for each agent.
			//in each game the words used - when in the list -
			//are updated.
			//The files thus keep track of the evolution of scores.
			if (parameters.getPrintScore()){

			    if (!(parameters.getDir()).equals("null")) 
				Utils.error(
				"if you want to print scores, please set the workDir parameter.");

			    String us=speaker.getUtterance();
			    String uh=hearer.getUtterance();
			    if (us!=null){
				if (lWords.contains(us) || lWords.contains(us)){
				    if (lWords.contains(us)){
					ns=((ArrayList)lWordList.get(s)).indexOf(us);
					if (ns<0){
					    ns=((ArrayList)lWordList.get(s)).size();
					    ((ArrayList)lWordList.get(s)).add(new String(us));
					    ((ArrayList)lScoreFiles.get(s)).
						add(new PrintWriter(new 
						        BufferedWriter(new FileWriter(
							parameters.getDir()+"/scoreA"+
							speaker.getID()+us+".txt"))));
					}
					speaker.
					    printScore((PrintWriter)((ArrayList)
						       lScoreFiles.get(s)).get(ns),
						       parameters.getUpdateScore(),us,N);
				    }
				    if (lWords.contains(uh)){
					nh=((ArrayList)lWordList.get(h)).indexOf(uh);
					if (nh<0){
					    nh=((ArrayList)lWordList.get(h)).size();
					    ((ArrayList)lWordList.get(h)).add(new String(uh));
					    ((ArrayList)lScoreFiles.get(h)).
						add(new PrintWriter(new BufferedWriter(new 
						    FileWriter(parameters.getDir()+"/scoreA"+
							       hearer.getID()+uh+".txt"))));
					}
					hearer.printScore((PrintWriter)((ArrayList)lScoreFiles.
							  get(h)).get(nh),
							  parameters.getUpdateScore(),us,N);
				    }
				}
				else if (lWords.size()<10 && us.equals(uh)){
				    //add this word only in case of success lg
				    lWords.add(new String(us));
				    ns=((ArrayList)lWordList.get(s)).size();
				    ((ArrayList)lWordList.get(s)).add(new String(us));
				    ((ArrayList)lScoreFiles.get(s)).
					add(new PrintWriter(new BufferedWriter(
					    new FileWriter(parameters.getDir()+"/scoreA"+
							   speaker.getID()+us+".txt"))));
				    System.out.println("added file: "+parameters.getDir()+"/scroreA"+
						       speaker.getID()+us+".txt");
				    speaker.printScore((PrintWriter)((ArrayList)
						       lScoreFiles.get(s)).get(ns),
						       parameters.getUpdateScore(),us,N);
				    nh=((ArrayList)lWordList.get(h)).size();
				    ((ArrayList)lWordList.get(h)).add(new String(uh));
				    ((ArrayList)lScoreFiles.get(h)).
					add(new PrintWriter(new BufferedWriter(
					   new FileWriter(parameters.getDir()+"/scoreA"+
							  hearer.getID()+uh+".txt"))));
				    System.out.println("added file: "+parameters.getDir()+"/scroreA"+
						       hearer.getID()+uh+".txt");
				    hearer.printScore((PrintWriter)((ArrayList)
						      lScoreFiles.get(h)).get(nh),
						      parameters.getUpdateScore(),uh,N);
				}
			    }
			}

			//if we want to output each language game
			if (parameters.getPrintGame()) pStats.print();

			//if we want to keep track of communicative success and so on.
			//in case of lexicon formation (holistic types) also keeps
			//track of individual games (utterances and meanings)
			if (parameters.getLogFile()){
			    if (logFile==null && !openedLogFile && 
				parameters.getVisible()){
				logFile = canvas.openLogFile();
			    }
			    
			    if (parameters.getFile()==null)
				if (logFile != null)
				    parameters.openFile(logFile);
				else
				    parameters.
					openFile(parameters.getLogFileName());
			    if (parameters.getFile()!=null){
				    pStats.logFile(parameters.getFile());
				    if (n%100==99)
					parameters.flushFile();
			    }
			    else{
				parameters.setLogFile(false);
				System.out.println("No logfile opened");
				System.exit(1);
			    }
			}
			//output to UI
			if (parameters.getVisible()){		    
			    canvas.updateGame(speaker,cxt,n,
					      hearer,iter,
					      parameters.getGameType(),
					      parameters.getAgentType(),pStats);
			}
			else if (n%1000==0 && (parameters.getAgentType()==1 ||
					       parameters.getAgentType()==0)){ 
			    //Additional output to standard out.
			    //"lx" prints lexicon sizes for each agent.
			    System.out.print("lx "+n+" ");
			    for (int i=0;i<lAgents.size();i++)
				if (parameters.getAgentType()==1)
				    System.out.print(((HolisticAgent2)lAgents.get(i)).
						     getLexSize()+" ");
				else System.out.print(((HolisticAgent)lAgents.get(i)).
						      getLexSize()+" ");
			    System.out.println();
			    //"on" prints ontology sizes for each agent
			    System.out.print("on "+n+" ");
			    for (int i=0;i<lAgents.size();i++) 
				if (parameters.getAgentType()==1)
				    System.out.print(((HolisticAgent2)lAgents.get(i)).
						     getOntSize()+" ");
				else System.out.print(((HolisticAgent)lAgents.get(i)).
						      getLexSize()+" ");
			    System.out.println();
			}

			if (parameters.getAgentType()==0 ||
			    parameters.getAgentType()==1){
			    //for the time being static prototypes are used in 
			    //compositional studies
			    //in all other cases, the prototypes move.
			    if (n%(lAgents.size()*100)==0 && n>1)
				for (int i=0;i<lAgents.size();i++)
				    ((Agent)lAgents.get(i)).
					merge((parameters.getFeatures()).length,
					      parameters.getUpdateScore());
			}
			//finished game, next one
			N++;
			
			//At the end of an iteration, we might want to test the population
			//or we might want to save the lexicon or grammar.
			if (N%parameters.getNRuns()==0){
			    Agent oldSpeaker=speaker;
			    Agent oldHearer=hearer;
			    iter++;
			    if (parameters.getTestPopulation()){ 
				testPopulation(iter);
				if (parameters.getVisible())				
				    canvas.updateGame(oldSpeaker,cxt,n,
						      oldHearer,iter,
						      parameters.getGameType(),
						      parameters.getAgentType(),pStats);
			    }
			    if (parameters.getSaveLexicon()){
				if (parameters.getLexIIFile()==null &&
				    parameters.getVisible()){
				    File file = canvas.openLexFile();
				    parameters.openLexIIFile(file);
				}
				if (parameters.getLexIIFile()!=null)
				    if (parameters.getAgentType()==2)
					Utils.printGrammar(lAgents,
							   parameters.getLexIIFile(),
							   parameters.getUpdateScore(),
							   iter);
				    else
					Utils.printGlobalLexicon(lAgents,
								 parameters.getLexIIFile(),
								 parameters.getUpdateScore(),
								 iter);
			    }
			    else if (parameters.getAgentType()==2)
				for (int i=0;i<lAgents.size();i++) 
				    ((CompositionalAgent2)lAgents.get(i)).printGrammarStats();

			    if (parameters.getPrintLexicon()){
				if (parameters.getLexFile()==null && 
				    parameters.getVisible()){
				    File file = canvas.openLexFile();
				    parameters.openLexFile(file);
				}
				if (parameters.getLexFile()!=null){
				    if (parameters.getAgentType()==2)
					Utils.printGrammar(lAgents,
							   parameters.getLexIIFile(),
							   parameters.getUpdateScore(),
							   iter);
				    else{
					Utils.printGlobalLexicon(lAgents,parameters.getLexFile(),
								 parameters.getUpdateScore(),iter);
					for (int i=0;i<lAgents.size();i++)
					    ((Agent)lAgents.get(i)).print(parameters.getLexFile(),
									  parameters.
									  getUpdateScore());
				    }
				    if (parameters.getVisible())
					canvas.savedLexFile();
				    
				}
			    }
			}			
		    }
		    //make sure, the simulation stops when necessary
		    if (parameters.getStep()){
			parameters.setStop(true);
			parameters.setStep(false);
		    }
		}
		if (parameters.getSaveLexicon()){
		    parameters.closeLexIIFile();

		    if (parameters.getVisible())
			canvas.savedLexFile();
		}
	    }

	    parameters.setRun(false);
	    parameters.setStop(false);
	    //other ways to print your lexicon.
	    if (parameters.getPrintLexicon() && N>0){

		if (parameters.getLexFile()==null && 
		    parameters.getVisible()){
		    File file = canvas.openLexFile();
			parameters.openLexFile(file);
		}
		if (parameters.getLexFile()!=null){
		    Utils.printGlobalLexicon(lAgents,parameters.getLexFile(),
					     parameters.getUpdateScore(),9999);
		    for (int i=0;i<lAgents.size();i++)
			((Agent)lAgents.get(i)).print(parameters.getLexFile(),
						      parameters.
						      getUpdateScore());
		    parameters.closeLexFile();
		    if (parameters.getVisible())
			canvas.savedLexFile();
		}

		parameters.setPrintLexicon(false);
	    }
	    if (!parameters.getVisible()){
		parameters.setQuit(true);
		System.out.println();
	    }
	}
	parameters.closeFile();
	System.exit(0);
    }
}



